﻿Type=Class
Version=2.2
B4J=true
@EndOfDesignText@
' Class module using DBUtils
' This class creates text fields above each column in a table view, which has been populated from a database.
' The class has an event for text changes in the text fields. Whenever a text field has changed, the table view is
' reloaded from the database with a filter as a where clause in the DBUtils call. The filter is constructed by combining
' the content of the text fields. When initializing the class, every column in the table view is marked as either "num" or "str".
'  - Columns of the type "num" are filtered using the filter text as it is, so if you enter ">2" in a "num" column filter, the content
'    of the column is assumed to be numeric and ">2" is used as a filter.
'  - Columns of the type "str" are filtered using the keyword "LIKE" followed by the filter text, with a "%" appended.

Sub Class_Globals
	Private fx As JFX
	Private filter_TableView As TableView
	Private filter_Db As SQL
	Private filter_Table As String
	Private filter_ColumnName(20) As String
	Private filter_ColumnNameString As String
	Private filter_ColumnType(20) As String
	Private txtFilter(20) As TextField
End Sub
' Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize (t As TableView, db As SQL, table As String, column() As String, columnType() As String, p As Pane)
	' Assign values to the global variables
	filter_TableView = t
	filter_Db = db
	filter_Table = table
	For i = 0 To t.ColumnsCount - 1
		filter_ColumnName(i) = column(i)
		filter_ColumnType(i) = columnType(i)
		filter_ColumnNameString = filter_ColumnNameString & column(i) & ", "
	Next
	filter_ColumnNameString = filter_ColumnNameString.SubString2(0,filter_ColumnNameString.Length - 2) 	' Remove the trailing comma sign from filter_ColumnNameString
	' Draw the filter text boxes above the table
	Dim x As Double
	Dim y As Double
	Dim w As Double
	x = t.Left
	y = t.Top - 24
	For i = 0 To t.ColumnsCount - 1
		x = x + w
		w = t.GetColumnWidth(i)
		txtFilter(i).Initialize("f_action")
		txtFilter(i).Id = "col" & i
		p.AddNode(txtFilter(i),x,y,w,24)
		txtFilter(i).PromptText = "Filter..."
	Next
End Sub
' Apply filter whenever a filter text field is changed
Private Sub f_action_TextChanged (old As String, new As String)
	Dim filterSQL As String ' The string filterSQL will hold the SQL WHERE clause
	' Loop through the table view columns and build the filterSQL
	For i = 0 To filter_TableView.ColumnsCount - 1
		' Check if the filter text field above this column contains anything
		If txtFilter(i).Text.Length > 0 Then 
			' Make sure that the filter string starts with "WHERE"
			If filterSQL = "" Then
				filterSQL = " WHERE " 
			End If
			' Append the content of the filter text field to filterSQL, taking the column type into account
			If filter_ColumnType(i) = "num" Then
				filterSQL = filterSQL & filter_ColumnName(i) & txtFilter(i).Text & " AND "
			Else
				filterSQL = filterSQL & filter_ColumnName(i) & " LIKE '" &txtFilter(i).Text & "%' AND "
			End If
		End If
	Next
	' Remove the trailing "AND" or "%' AND" from the filter
	If filterSQL.Length > 0 Then
		' Check what to remove
		If filterSQL.SubString2(filterSQL.Length - 6, filterSQL.Length - 5) = "'" Then
			filterSQL = filterSQL.SubString2(0,filterSQL.Length - 6) & "'"
		Else
			filterSQL = filterSQL.SubString2(0,filterSQL.Length - 5)
		End If
	End If
	' Ignore errors caused by incorrect SQL requests
	Try
		DBUtils.ExecuteTableView(filter_Db, "SELECT " & filter_ColumnNameString & " FROM " & filter_Table & filterSQL, Null, 0, filter_TableView)
	Catch
		Log(LastException.Message)
	End Try
End Sub